@@ -20,9 +20,11 @@ import sys
 from IPython.parallel import Client
 from IPython.parallel.apps import launcher
 from IPython.parallel import error as iperror
+from IPython.utils.path import locate_profile, get_security_file
 from IPython.utils import traitlets
 from IPython.utils.traitlets import (List, Unicode, CRegExp)
 
+
 # ## Custom launchers
 
 # Handles longer timeouts for startup and shutdown
@@ -139,14 +141,12 @@ def _find_parallel_environment():
                      "See %s for SGE setup instructions." %
                      "https://blogs.oracle.com/templedf/entry/configuring_a_new_parallel_environment")
 
-
 def _has_parallel_environment(line):
     if line.startswith("allocation_rule"):
         if line.find("$pe_slots") >= 0 or line.find("$fill_up") >= 0:
                 return True
     return False
 
-
 # ## SLURM
 class SLURMLauncher(launcher.BatchSystemLauncher):
     """A BatchSystemLauncher subclass for SLURM
@@ -230,7 +230,7 @@ srun -N {machines} -n {n} %s %s --profile-dir="{profile_dir}" --cluster-id="{clu
         self.context["machines"] = self.machines
         self.context["account"] = self.account
         self.context["timelimit"] = self.timelimit
-        return super(BcbioSLURMEngineSetLauncher, self).start(n)
+        return super(BcbioOLDSLURMEngineSetLauncher, self).start(n)
 
 
 class BcbioOLDSLURMControllerLauncher(SLURMLauncher, launcher.BatchClusterAppMixin):
@@ -252,7 +252,7 @@ class BcbioOLDSLURMControllerLauncher(SLURMLauncher, launcher.BatchClusterAppMix
         """Start the controller by profile or profile_dir."""
         self.context["account"] = self.account
         self.context["timelimit"] = self.timelimit
-        return super(BcbioSLURMControllerLauncher, self).start(1)
+        return super(BcbioOLDSLURMControllerLauncher, self).start(1)
 
 
 # ## PBS
@@ -408,6 +408,25 @@ def _get_profile_args(profile):
     else:
         return ["--profile=%s" % profile]
 
+def _scheduler_resources(scheduler, params):
+    """Retrieve custom resource tweaks for specific schedulers.
+    Handles SGE parallel environments, which allow multicore jobs
+    but are specific to different environments.
+    """
+    resources = params.get("resources", [])
+    if isinstance(resources, basestring):
+        resources = resources.split(";")
+    if scheduler in ["SGE"]:
+        pename = None
+        for r in resources:
+            if r.startswith("pename="):
+                _, pename = r.split("=")
+                resources.remove(r)
+                break
+        if pename is None:
+            pename = _find_parallel_environment()
+
+    return ";".join(resources), pename
 
 def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
            extra_params):
@@ -426,6 +445,7 @@ def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
                "your scheduler. If it should, please file a bug report at "
                "http://github.com/roryk/ipython-cluster-helper. Thanks!")
         sys.exit(1)
+    resources, pename = _scheduler_resources(scheduler, extra_params)
 
     args = launcher.ipcluster_cmd_argv + \
         ["start",
@@ -436,7 +456,7 @@ def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
          "--debug",
          "--n=%s" % num_jobs,
          "--%s.cores=%s" % (engine_class, cores_per_job),
-         "--%s.resources=%s" % (engine_class, extra_params.get("resources", "")),
+         "--%s.resources='%s'" % (engine_class, resources),
          "--%s.mem='%s'" % (engine_class, extra_params.get("mem", "")),
          "--IPClusterStart.controller_launcher_class=%s.%s" % (ns, controller_class),
          "--IPClusterStart.engine_launcher_class=%s.%s" % (ns, engine_class),
@@ -444,24 +464,21 @@ def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
          "--cluster-id=%s" % (cluster_id)
          ]
     args += _get_profile_args(profile)
-    if scheduler in ["SGE"]:
-        args += ["--%s.pename=%s" % (engine_class, _find_parallel_environment())]
-    elif scheduler in ["SLURM"]:
-        if _slurm_is_old():
-            # SLURM cannot get resource atts (native specification) as SGE does
-            slurm_atrs = {}
-            extra_params = extra_params['resources'].split(';')
+    if pename:
+        args += ["--%s.pename=%s" % (engine_class, pename)]
+    elif scheduler in ["OLDSLURM"]:
+        # SLURM cannot get resource atts (native specification) as SGE does
+        slurm_atrs = {}
+        for parm in resources.split(";"):
+            atr = parm.split('=')
+            slurm_atrs[atr[0]] = atr[1]
+        extra_params = slurm_atrs
 
-            for parm in extra_params:
-                atr = parm.split('=')
-                slurm_atrs[atr[0]] = atr[1]
-            extra_params = slurm_atrs
-
-            args += ["--%s.machines=%s" % (engine_class, extra_params.get("machines", "1"))]
-            args += ["--%s.account=%s" % (engine_class, extra_params["account"])]
-            args += ["--%s.account=%s" % (controller_class, extra_params["account"])]
-            args += ["--%s.timelimit=%s" % (engine_class, extra_params["timelimit"])]
-            args += ["--%s.timelimit=%s" % (controller_class, extra_params["timelimit"])]
+        args += ["--%s.machines=%s" % (engine_class, extra_params.get("machines", "1"))]
+        args += ["--%s.account=%s" % (engine_class, extra_params["account"])]
+        args += ["--%s.account=%s" % (controller_class, extra_params["account"])]
+        args += ["--%s.timelimit=%s" % (engine_class, extra_params["timelimit"])]
+        args += ["--%s.timelimit=%s" % (controller_class, extra_params["timelimit"])]
 
     subprocess.check_call(args)
     return cluster_id
@@ -566,30 +583,19 @@ def create_throwaway_profile():
     subprocess.check_call(cmd, shell=True)
     return profile
 
-
-def get_ipython_dir(profile):
-    proc = subprocess.Popen("ipython locate", stdout=subprocess.PIPE, shell=True)
-    ipython_dir = proc.stdout.read().strip()
-    profile_dir = "profile_{0}".format(profile)
-    return os.path.join(ipython_dir, profile_dir)
-
-
 def get_url_file(profile, cluster_id):
-    if os.path.isdir(profile) and os.path.isabs(profile):
-        ipython_dir = profile
-    else:
-        ipython_dir = get_ipython_dir(profile)
-    security_dir = os.path.join(ipython_dir, "security")
-    url_file = "ipcontroller-{0}-client.json".format(cluster_id)
-    return os.path.join(security_dir, url_file)
 
+    url_file = "ipcontroller-{0}-client.json".format(cluster_id)
+
+    if os.path.isdir(profile) and os.path.isabs(profile):
+        # Return full_path if one is given
+        return  os.path.join(profile, "security", url_file)
+
+    return os.path.join(locate_profile(profile), "security", url_file)
 
 def delete_profile(profile):
     MAX_TRIES = 10
-    proc = subprocess.Popen("ipython locate", stdout=subprocess.PIPE, shell=True)
-    ipython_dir = proc.stdout.read().strip()
-    profile_dir = "profile_{0}".format(profile)
-    dir_to_remove = os.path.join(ipython_dir, profile_dir)
+    dir_to_remove = locate_profile(profile)
     if os.path.exists(dir_to_remove):
         num_tries = 0
         while True:
